/* ISM.c           ---------------------------------------------- */
/* Interactive Spool Manager - Gestore code di spool in Unix/Aix  */
/*                                                                */
/* - Visualizza code e stampe relative                            */
/* - Permette cancellazione/visione/modifica delle stampe         */
/* - Permette di dare comandi relativi alla coda selezionata      */
/*                                                                */
/*                                                                */
/* Note:                                                          */
/*     Compilazione:       cc ism.c -oism -lcurses -DAIX          */
/*                                                                */
/*     Necessita del programma 'most' per visualizzare le stampe  */
/*     Necessita del programma 'pt' per far ripartire la stampa   */
/*         da una determinata pagina                              */
/*                                                    Piero  2/98 */
/* -------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>

#ifdef AIX
   #include <curses.h>                  /* ambiente Unix standard  */
#else
   #include <ncurses/curses.h>          /* ambiente Linux */
#endif

#define  MAXLINE  200           /* lunghezza massima riga di nome file  */
#define  MAXSTR   80            /* lunghezza massima stringa */

#define  MAXQUE   100           /* numero massimo di code    */
#define  MAXJOB   500           /* numero massimo di stampe x coda */

/* Variabili globali ----------------------------- */

char filetemp[MAXSTR], comando[MAXSTR];
char record[MAXLINE];

char filelog[MAXSTR] = "ism.log";

FILE *ifp;             /* file temporaneo output lpstat                 */
FILE *lfp;             /* eventuale file di log in output accodamento   */
FILE *filepr;          /* file di stampa       */
FILE *qconfig;         /* file configurazione code /etc/qconfig         */
int numrighe = 25;     /* numero di righe dello schermo                 */
int numcol   = 80;     /* numero di colonne dello schermo               */
int r1, c1, r2, c2;    /* righe e colonne                               */
chtype chc0[10];       /* caratteri della cornice attuale               */
  /* tipo di cornice n. 1 */
chtype chc1[10] = {'|', '-', '+', '+', '+', '+', '|', '|', '-', '-'};
  /* tipo di cornice n. 2       */
chtype chc2[10] = {0x000000b3,0x000000c4,0x000000da,0x000000bf,0x000000c0,
       0x000000d9,0x000000c3,0x000000b4,0x000000c2,0x000000c1};

char typec = 1;           /* indicatore tipo di cornice                 */
char fillbox = 0;         /* = 0 per comando BOX, = 1 per WINDOW        */
chtype attr, mask;        /* per gestione attributi                     */

int  gsett, gmese, mese;  /* giorno della settimana, del mese e num. mese */
char ora[6], anno[6];     /* ore:minuti, anno                           */
char dateday[9];          /* data nella forma gg/mm/aa                  */
char minsec[6];           /* stringa contenente minuti e secondi        */

char workq[20];           /* coda attuale                               */
int  nworkq;              /* indice di coda attuale                     */
char fileprint[MAXSTR];   /* path e nome file del file in stampa        */
int  nlinepag = 20;       /* num. massimo linee di elenco per videata   */

int  totq;                /* numero totale di code nel sistema          */
int  totfileq;            /* numero totale di stampe in coda attuale    */
int  RC = 0;              /* Return Code dei comandi 'enq'              */
char tipovis = 'C';       /* 'C' per elenco code, 'J' per elenco stampe */
char logflag = ' ';       /* 'S' se e' richiesto il file di log         */
char queflag = ' ';       /* 'S' se e' richiesta restrizione nomi code  */
char fulflag = ' ';       /* 'S' se e' richiesta statistica iniziale code*/
char X3flag  = ' ';       /* 'S' se e' richiesta compatibilita' con AIX 3*/
char bflag   = ' ';       /* 'S' se e' richiesta analisi qconfig        */
char rigen   = ' ';       /* 'S' se rigenera report                     */
char nomeuser[16];        /* nome utente                                */
char hostname[16];        /* nome host                                  */
char homedir[MAXSTR];     /* home directory                             */
char pagprint[10];        /* pagine attualmente stampate                */
char elencocmd[16];       /* elenco comandi proibiti                    */
char elencoque[MAXSTR];   /* elenco code permesse                       */
char elencobak[MAXSTR];   /* elenco backend code permessi               */
char backend[6][64];      /* elenco singoli backend permessi            */
int  nbackend = 0;        /* num. singoli backend                       */
char lineablank[MAXSTR];
char titolo[30];          /* Titolo videata                             */
int  tagged   = 0;        /* Numero stampe selezionate con barra spazio */

/* Campi per dati statistici file di stampa   */
long int filelen, numctrll, numpag, numlines, maxlenrec, linexpag = 66;
struct stat statbuf;

/* Campi per ripartenza file di stampa   */
char pagini[10], pagfin[10];

/* Campi per comando 'G'                     */
char cp_copie[4];            /* numero di copie (1-99)   [-N99] */
char cp_hold[4];             /* Hold stampa (S/N)     */
char cp_top[4];              /* margine alto in n. linee [-t99] */
char cp_left[4];             /* margine sinistro in car. [-i99] */
char cp_lpage[8];            /* lunghezza pagina in linee[-l9999] */
char cp_compr[4];            /* stampa compressa (S/N)   [-K+!] */
char cp_enf[4];              /* stampa enfatizzata (S/N) [-e+!] */
char cp_font[22];            /* nome font stampante laser [-s..] */
char cp_pitch[4];            /* grandezza font stampante laser [-p99] */
char cp_lpi[4];              /* linee per inch stampante laser [-v99] */
char cp_duplex[2];           /* Solo fronte o fronte/retro (F/L/C) [-Y012] */
char cp_form[2];             /* Portrait o Landscape (P/L) [-z01] */
char cp_binout[2];           /* Fascicolatore in output (0-99) [-=99] */
char cp_source[2];           /* Paper Source: 1=prim. 2=Alt. 3=buste [-u] */
char cp_cmd[50];             /* parametri del comando Unix qprt risultante */

char cp_yn[2] ;              /* Campo di conferma e di uscita  */

/* Campi raccolti con 'lsallq -c'            */
char qname[MAXQUE][20];      /* 100 nomi code lunghi al massimo 20      */
char qdev[MAXQUE][20];       /* 100 nomi device lunghi al massimo 20    */
char qstatus[MAXQUE][10];    /* stato della coda                        */
char qremstat[MAXQUE][10];   /* stato della coda remota                 */
int  qnum[MAXQUE];           /* numero dei file nella coda              */

/* Campi raccolti con 'qchk -L -P'            */
char filetag[MAXJOB];        /* segnalatore tag di selezione stampa     */
char filename[MAXJOB][20];   /* nome file di stampa                     */
char filepath[MAXJOB][70];   /* path e nome file fisico di stampa       */
char filedate[MAXJOB][15];   /* data e ora di produzione stampa         */
char filejob[MAXJOB][5];     /* job number stampa                       */
char filepri[MAXJOB][5];     /* priorita' di stampa                     */
char fileblk[MAXJOB][8];     /* numero di blocchi                       */
char filecpy[MAXJOB][5];     /* numero di copie                         */
char filestat[MAXJOB][10];   /* stato della stampa                      */
char fileuser[MAXJOB][16];   /* utente che ha prodotto la stampa        */
char fileuser1[MAXJOB][16];  /* utente To                               */


/* Stringhe impostabili con il Setup                      */
char CLEAR[8] = "clear ";
char EDITOR1[8] = "vi ";
char EDITOR2[8] = "x ";
#ifdef AIX
   char COMMD1[20] = "lsallq -c|grep : ";
   char COMMD2[20] = "qchk -L -P";
   char PAGER1[8] = "most ";     /* "most ";   */
   char PAGER2[8] = "view ";
#else
   char COMMD1[20] = "lpstat";
   char COMMD2[20] = "lpstat";
   char PAGER1[8] = "less ";
   char PAGER2[8] = "less ";
#endif

void hlp_scr(); /* prototipo della funzione help screen che e' in fondo */
void hlp_par(); /* prototipo della funzione help parametri in fondo */

/* Display linea in riga 23 */
void display23 (char *str)
{
    int i;

    for (i = 0; i < 80 ; i++) mvaddch(23, i, ' ');
    attrset (A_REVERSE);
    mvaddstr (23, 0, str);
    refresh();
    attrset (A_NORMAL);
}


/* Scrive una cornice in stdscr; input: r1,c1,r2,c2, typec, fillbox  */
void cornice(int r1, int c1, int r2, int c2)
{
    int i,j;

    if (typec == 1)  for (i=0;i<=9;i++) chc0[i] = chc1[i];
    if (typec == 2) {
       for (i=0;i<=9;i++) chc0[i] = chc2[i];
       attr = A_ALTCHARSET ;
       attrset (attr);
    }

    /* per ogni riga della finestra da delimitare     */
    for (i = r1 ; i <= r2 ; i++ ) {
       if (i == r1) mvaddch(r1, c1, chc0[2] );
       else if (i == r2) mvaddch(r2, c1, chc0[4]);
            else mvaddch(i, c1, chc0[0] );

       /* fa il bordo o il centro     */
       for (j = c1+1 ; j <= c2-1 ; j++ ) {
           if ((i == r1) || (i == r2)) addch(chc0[1] );
           else  if ( fillbox == 1)  addch(' ');
       }

       /* completa il bordo destro */
       if (i == r1) mvaddch(r1, c2, chc0[3] );
       else if (i == r2) mvaddch(r2, c2, chc0[5] );
            else mvaddch(i, c2, chc0[0] );

    }

    attrset (A_NORMAL );

}

/* Scrive il messaggio 'ATTENDERE' in mezzo al video   */
void attendere()
{
    char stringa[26] =   "                        ";
    char attendere[26] = "   A T T E N D E R E  ! ";

    attrset (A_NORMAL );
    cornice( 12, 26, 16, 52);
    mvaddstr(13, 27, stringa);
    mvaddstr(14, 27, attendere);
    mvaddstr(15, 27, stringa);

    refresh();

}

/* Estrae data e ora        */
/* Output: gsett, mese, gmese, ora, anno, dateday, minsec     */
void data()
{
    time_t t;
    char wd[22] = "SunMonTueWedThuFriSat";
    char mo[38] = "JanFebMarAprMayJunJulAugSepOctNovDec";
    char s[30], ss[4] ; int i;

    time(&t);             /* estrae timestamp    */
    strcpy (s,ctime(&t)); /* trasforma timestamp in stringa: Set Mes nn hh:mm:ss yyyy */

    strncpy (ss, s, 3); ss[3] = 0;
    gsett = ( strstr(wd, ss) - wd ) / 3; gsett++;  /* 1 = Domenica   */
    strncpy (ss, s+4, 3);
    mese = ( strstr(mo, ss) - mo ) / 3;  mese++;
    strncpy (ss, s+8, 3);
    gmese =  atoi(ss); strncpy(ora, s+11, 5); strncpy(anno, s+20, 4);
    if ((gsett<1) || (gsett>7)) gsett = 0;
    if ((mese<1) || (mese>12)) mese = 0;
    sprintf(dateday, "%2d/%02d/%c%c", gmese, mese, anno[2], anno[3]);
    strncpy(minsec, s+14, 2); strncpy(minsec+2, s+17, 2); minsec[4] = 0;

}

/* Produce l'elenco delle code presenti nel sistema               */
/*     raccogliendo tutti i dati nei vettori opportuni (q*[])     */

void getqueues()
{
    int  nrec;     /* contatore dei record letti          */
    int  indq = 0;
    int  i, j, k, l;
    char comando[MAXSTR];
    char *ptr;

    strcpy(comando, COMMD1);     /* lsallq | grep :  */
    strcat(comando, " > ");
    strcat(comando, filetemp);
    if ( system(comando) != 0) {
       fprintf(stderr,"ISM - Errore in comando: %s.\n", comando);
       exit (98);
    }

    if ( (ifp = fopen(filetemp, "r")) == NULL) {
       fprintf(stderr,"ISM - Errore in apertura file %s.\n",filetemp);
       exit (97);
    }

    nrec = 0;
    /* legge il file delle code fino alla fine */
    while ( fgets(record, MAXLINE, ifp) != NULL ) {
       nrec ++;
       ptr = strchr(record, ':');
       strncpy(qname[nrec],record,ptr-record);
       qname[nrec][ptr-record] = '\0';
       strcpy(qdev[nrec],ptr+1);
       l = strlen(qdev[nrec]);
       qdev[nrec][l-1] = '\0';
       strcpy(qstatus[nrec],"");
       strcpy(qremstat[nrec],"");
       qnum[nrec] = -1;
       if ((queflag == 'S') && ( bflag != 'S') && (strstr(elencoque,qname[nrec])==NULL) ) {
          /* La gestione di questa coda non e' permessa   */
          nrec--;
       }
    }
    fclose(ifp);

    totq = nrec;

    /* Se vi e' selezione per backend, toglie le code in piu'   */
    if ( bflag == 'S' ) {    /*  Apre il file /etc/qconfig */
       if ( (qconfig = fopen("/etc/qconfig", "r")) == NULL) {
          fprintf(stderr,"ISM - Errore in apertura '/etc/qconfig'.\n");
          exit (97);
       }
       /* legge il file 'qconfig' fino alla fine */
       indq = -1;
       while ( fgets(record, MAXLINE, qconfig) != NULL ) {
          if ((record[0] != '*') && (record[0] != '#')) {
             for (i = 1; i <= totq; i++)  if (strstr(record,qdev[i]) == record) indq = i;
             if (strstr(record,"backend") != NULL) {
                strcpy(record,record+8);
                for (j = 0; j <= nbackend; j++) {
                   if (strstr(record, backend[j]) != NULL)  break;
                }
                if ((j > nbackend) || ((queflag == 'S') && (strstr(elencoque,qname[indq])==NULL))) {
                   for (k = indq; k < totq; k++) {
                      strcpy(qname[k], qname[k+1]);
                      strcpy(qdev[k],  qdev[k+1]);
                   }
                   totq--;
                }
             }
          }
       }
       fclose(qconfig);
    }

}


/* Produce l'elenco delle stampe presenti nella coda specificata  */
/*     raccogliendo tutti i dati nei vettori opportuni (file*[])  */

void getspool(char *nomecoda)
{
    char comando[MAXSTR], q[8], q1[8], s[10], d[10];
    char *ptr;
    int  nrec;     /* contatore dei record letti          */
    int  l,i;

    attendere();

    strcpy(comando, COMMD2);     /* qchk -L    */
    strcat(comando, nomecoda);
    strcat(comando, " > ");
    strcat(comando, filetemp);
    if ( system(comando) != 0) {
       fprintf(stderr,"ISM - Errore in comando: %s.\n", comando);
       exit (98);
    }

    if ( (ifp = fopen(filetemp, "r")) == NULL) {
       fprintf(stderr,"ISM - Errore in apertura file %s.\n",filetemp);
       exit (97);
    }

    tagged = 0;
    nrec = 0;
    strcpy(q1, ".......");
    strcpy(pagprint, "      ");
    /* salta i primi tre record di intestazione          */
    fgets(record, MAXLINE, ifp);
    fgets(record, MAXLINE, ifp);
    fgets(record, MAXLINE, ifp);
    /* legge il file delle stampe in coda fino alla fine */
    while ( fgets(record, MAXLINE, ifp) != NULL ) {

       l = strlen(record);
       strncpy(record+l-1,lineablank,70);

       strncpy(q,record,7);    q[7] = '\0';
       strncpy(s,record+15,9); s[9] = '\0';
       strncpy(d,record+24,6); d[6] = '\0';

       if ( strncmp(q, "       ", 6) != 0) { /* nome coda   */
          if ( strncmp(q, q1 , 7) == 0) {
             strcpy(qremstat[nworkq], s);
             break;
          }
          else {
             nrec = 0;
             /* azzeramento vettore */
             strcpy(filename[nrec],"");
             strcpy(filepath[nrec],"");
             strcpy(filedate[nrec],"");
             strcpy(filejob[nrec],"");
             strcpy(filepri[nrec],"");
             strcpy(fileblk[nrec],"");
             strcpy(filecpy[nrec],"");
             strcpy(filestat[nrec],"");
             strcpy(fileuser[nrec],"");
             strcpy(fileuser1[nrec],"");
             strcpy(filepath[nrec],"");
             filetag[nrec] = ' ';
             strcpy(qstatus[nworkq], s);
             strcpy(q1, q);
          }
       }
       if ( (strncmp(s, "        ", 8) != 0) &&
            (strncmp(d, "      ", 5) != 0) )  {  /* job in coda */
          nrec++;
          filetag[nrec] = ' ';
          strcpy(filestat[nrec], s);
          strncpy(filejob[nrec],record+24,4);   filejob[nrec][4] = '\0';
          strncpy(filename[nrec],record+33,14); filename[nrec][14]='\0';
          strncpy(fileuser[nrec],record+48,14); fileuser[nrec][14]='\0';
          strncpy(fileuser1[nrec],record+63,14); fileuser1[nrec][14]='\0';
       }
       else
          if ( strncmp(d, "      ", 5) != 0) {  /* seconda linea */
             strncpy(filedate[nrec],record+24,14); filedate[nrec][14]='\0';
             strncpy(filepri[nrec],record+48,4); filepri[nrec][4] = '\0';
             strncpy(fileblk[nrec],record+55,6); fileblk[nrec][6] = '\0';
             strncpy(filecpy[nrec],record+62,4); filecpy[nrec][4] = '\0';
             if (strcmp(pagprint, "      ") == 0) {
                strncpy(pagprint, record+73,7);
                pagprint[7] = '\0';
             }
          }
          else  {                          /* terza linea  */
                strncpy(filepath[nrec],record+31,50); filepath[nrec][50]='\0';
                for (i = 2; i < 50 ; i++)
                    if (filepath[nrec][i] == ' ') {
                       filepath[nrec][i] = '\0';
                       break;
                    }
                };

       strcpy(record, lineablank);
    }
    fclose(ifp);

    totfileq = nrec;
    qnum[nworkq] = totfileq ;

    clear();
}

/* Riempie la schermata da vettori file*[] oppure q*[] in         */
/*         funzione del valore di tipovis                         */
/* In input ha la posizione del cursore sul video e nel vettore   */

void videoout(int linecurs, int nrec)
{
   char s[8], stringa[80];

   int  i, l, col = 2, riga = 0;

   /* Scrive 2 linee orizzontali */
   attrset(A_NORMAL);
   /* attrset(A_ALTCHARSET); */
   for (i = 0 ; i < 79; i++) {
      mvaddch(0, i, chc0[1]);
      mvaddch(2, i, chc0[1]);
   }

   /* Scrive cornici e intestazioni     */
   mvaddstr(0, 1, "User: ");
   mvaddstr(0, 7, nomeuser);
   mvaddstr(0, 65, dateday);
   mvaddstr(0, 74, ora);
   attrset(A_NORMAL | A_REVERSE);
   mvaddstr(0, 30, titolo);

   attrset (A_NORMAL | A_BOLD );
   mvaddstr ( 1, 1, lineablank );
   attrset (A_NORMAL);
   if (tipovis == 'C')  {    /* visualizzazione code  */
      sprintf(stringa, "%4d code ", totq);
      mvaddstr ( 1, 1, stringa);
   }
   else   {                  /* visualizzazione stampe    */
      sprintf(stringa, "Coda:                  Stato: %9s %9s                  %4d stampe",
           qstatus[nworkq], qremstat[nworkq], totfileq);
      mvaddstr ( 1, 1, stringa);
      if (strncmp(pagprint, "      ",6) != 0) {
         sprintf(stringa, "Pag.:%s%%",pagprint);
         mvaddstr ( 1, 51, stringa);
      }
      attrset (A_REVERSE);
      mvaddstr ( 1, 7, workq);
      if (strncmp(qstatus[nworkq],"DOWN ",4) == 0) {
         attrset (A_BLINK);
         mvaddstr ( 1, 31, "DOWN");
      }
   }
   attrset (A_NORMAL | A_BOLD);
   if (tipovis == 'C')  {    /* visualizzazione code  */
      mvaddstr ( 2, col, "Coda ");
      mvaddstr ( 2, col+15, "Device ");
      mvaddstr ( 2, col+30, "Stato ");
      mvaddstr ( 2, col+40, "Stato Remoto ");
      mvaddstr ( 2, col+70, "Stampe  ");
   }
   else {                    /* visualizzazione stampe    */
      mvaddstr ( 2, col, "Nome ");
      mvaddstr ( 2, col+15, "Stato ");
      mvaddstr ( 2, col+25, "Utente ");
      mvaddstr ( 2, col+40, "Data ");
      mvaddstr ( 2, col+49, "Ora ");
      mvaddstr ( 2, col+56, "Job Pri Copie  Blocchi");
   }

   l = nrec - linecurs + 1;
   for (i = 1 ; i <= nlinepag ; i++) {
       riga = i;
       attrset (A_NORMAL );
       if (tipovis == 'C') {     /* visualizzazione code  */
          if (l > totq ) break;
          if (i == linecurs)  attrset (A_REVERSE);
          else  attrset (A_NORMAL );
          mvaddstr(riga+2, 0, lineablank);
          mvaddstr(riga+2, col, qname[l]);
          mvaddstr(riga+2, col+15, qdev[l]);
          mvaddstr(riga+2, col+30, qstatus[l]);
          mvaddstr(riga+2, col+40, qremstat[l]);
          if (qnum[l] >= 0) {
             sprintf(s, "%4d", qnum[l]);
             mvaddstr(riga+2, col+72, s);
          }
       }
       else {                    /* visualizzazione stampe  */
          if (l > totfileq ) break;
          mvaddstr(riga+2, 0, lineablank);
          attrset (A_NORMAL | A_BOLD);
          mvaddch(riga+2, 0, filetag[l]);
          if (i == linecurs)  attrset (A_REVERSE);
          else  attrset (A_NORMAL );
          mvaddstr(riga+2, 1, lineablank);
          mvaddstr(riga+2, col, filename[l]);
          mvaddstr(riga+2, col+15, filestat[l]);
          mvaddstr(riga+2, col+25, fileuser[l]);
          mvaddstr(riga+2, col+40, filedate[l]);
          mvaddstr(riga+2, col+55, filejob[l]);
          mvaddstr(riga+2, col+61, filepri[l]);
          mvaddstr(riga+2, col+65, filecpy[l]);
          mvaddstr(riga+2, col+72, fileblk[l]);
       }
       l++;
   }
   attrset (A_REVERSE);
   if (tipovis == 'C')      /* visualizzazione code  */
   mvaddstr ( 23, 0, " Help = ?/F1    Scegli la coda da vedere e premi <Invio>        Fine = q / F10");
   else
   mvaddstr ( 23, 0, " Help = ?/F1    Comandi: u d v p s h r t x z c g i              Fine = q / F10");

   attrset (A_NORMAL );
   mvaddstr ( 1, 0, "");

 }

/* Input da tastiera di una stringa lunga 'l', con possibilita' di editing */
/* Si presuppone l'uso di curses, e che sia preventivamente dato initscr, cbreak */

int inputtext(char *str, int l, char tipo, char tipomap)
{
    /*  restituisce il carattere di uscita campo             */
    /*  'tipo' puo' essere: '9' solo cifre numeriche 0-9;    */
    /*                      'A' solo alfanumerico (lettere+numeri)  */
    /*                      'U' solo alfanumerico convertito in maiuscolo*/
    /*                          (i blank sono sempre permessi*/
    /*   campi al massimo lunghi 80 (l = lunghezza )         */
    /*  'tipomap' puo' essere: 'F' per mappa full-screen oppure qualunque*/

    int i, j, n;
    int c;
    char stringa[80];

    noecho();
    keypad (stdscr, TRUE);
    tipo = toupper(tipo);
    tipomap = toupper(tipomap);
    n = strlen(str);
    if ((l < 0) || (l > 80)) l = 80;
    for (i = 0 ; i < l ; i++) {
       if (i < n ) c = str[i];
       else c = ' ';
       addch (c);
       stringa[i] = c;
    }

    for (i = 0 ; i < l ; i++) addch('\b');

    i = 0;
  itera:
    refresh();
    c = getch();
    if (c == '\n') {  /* se e' premuto ENTER, esce */
       stringa[n] = 0;
       strcpy(str, stringa);
       echo();
       return c;
    }
    if (c == KEY_RIGHT) {
       if (i < l) {
          i++;
          addch(stringa[i-1]);
          goto itera;
       }
       else goto errore;
    }
    if (c == KEY_DC)  {   /* delete character */
       if (n > i) {
          n--;
          strcpy(stringa+i,stringa+i+1); stringa[l-1] = ' ';
          for (j = i ; j < l ; j++) addch(stringa[j]);
          for (j = i ; j < l ; j++) addch('\b');
          goto itera;
       }
       else goto errore;
    }
    if ((c == KEY_BACKSPACE) || (c == KEY_LEFT)) {
       if (i > 0) {
          i--;
          addch('\b');
          goto itera;
       } else goto errore;
    }
    if ((c == KEY_DOWN) || (c == KEY_UP) || (c == KEY_HOME) || (iscntrl(c)) ||
       (c == KEY_NPAGE) || (c == KEY_PPAGE) || (c == KEY_DC)) {
       if (tipomap == 'F') {
          stringa[n] = 0;
          strcpy(str, stringa);
          echo();
          return c;
       }
       else goto errore;
    }
    if (tipo == '9') {
        if (! (isdigit(c)) && (c != ' ') ) goto errore;
    }
    if ((tipo == 'A') && ! (isdigit(c) || isalpha(c) || c == ' ')) goto errore;
    if ((tipo == 'U') && ! (isdigit(c) || isalpha(c) || c == ' ')) goto errore;
    if (tipo == 'U') c = toupper(c);
    if (i >= l) goto errore;
    stringa[i] = c;
    i++;
    if (i > n) n = i;
    addch(c);
    goto itera;
  errore:
    beep();
    goto itera;
}

/* Gestione mappa Full Screen per cambio caratteristiche della stampa */
/* Restituisce il carattere di conferma o meno della mappa   */

int chg_stampa(int nrec)
{
   int i, c;
   i = nrec;

   /* Inizializzazione campi in input                   */
   strcpy(cp_copie," ");
   strcpy(cp_hold, " ");
   strcpy(cp_compr, " ");
   strcpy(cp_enf, " ");
   strcpy(cp_font, "         ");
   strcpy(cp_pitch, "  ");
   strcpy(cp_lpi, "  ");
   strcpy(cp_duplex, " ");
   strcpy(cp_form, " ");
   strcpy(cp_binout, " ");
   strcpy(cp_source, " ");
   strcpy(cp_top, "  ");
   strcpy(cp_left, "  ");
   strcpy(cp_lpage, "  ");
   strcpy(cp_cmd, " ");

   strcpy(cp_yn, " ");

   /* Scrittura sullo schermo dei campi in output fissi */
   clear();
   attrset (A_NORMAL);
   mvaddstr ( 0, 1, "----------------------------------------------------------------------------");
   mvaddstr ( 1, 1, "Nome stampa:");
   mvaddstr ( 1,45, "Stato" );
   mvaddstr ( 1,60, "Job number:" );
   mvaddstr ( 1,16, filename[i]);
   mvaddstr ( 1,55, filestat[i]);
   mvaddstr ( 1,72, filejob[i]);
   mvaddstr ( 2, 1, "----------------------------------------------------------------------------");

   attrset (A_NORMAL | A_BOLD);
   mvaddstr ( 3, 1, "Eventuale modifica delle caratteristiche della stampa da copiare:");

   mvaddstr ( 5, 1, "Hold Stampa (S/N):" );
   mvaddstr ( 6, 1, "Numero copie (max. 99):" );
   mvaddstr ( 7, 1, "Stampa compressa (S/N):");
   mvaddstr ( 7,56, "(solo stampanti impatto)");
   mvaddstr ( 8, 1, "Stampa enfatizzata (S/N):");
   mvaddstr ( 8,56, "(solo stampanti impatto)");
   mvaddstr (10, 1, "Nome Font da utilizzare:" );
   mvaddstr (10,56, "(solo stampanti laser)");
   mvaddstr (11, 1, "Caratteri per pollice:" );
   mvaddstr (11,56, "(solo stampanti laser)");
   mvaddstr (12, 1, "Numero linee per pollice:" );
   mvaddstr (12,56, "(solo stampanti laser)");
   mvaddstr (13, 1, "Fronte o fronteRetro (F/L/C):" );
   mvaddstr (13,56, "(solo stampanti laser)");
   mvaddstr (14, 1, "Portrait o Landscape (P/L):" );
   mvaddstr (14,56, "(solo stampanti laser)");
   mvaddstr (15, 1, "Tipo Modulo (1,2,3):" );
   mvaddstr (15,56, "(solo stampanti laser)");
   mvaddstr (16, 1, "Fascicolatore numero:" );
   mvaddstr (16,56, "(solo stampanti laser)");
   mvaddstr (18, 1, "Margine alto:" );
   mvaddstr (18,56, "(numero linee bianche)");
   mvaddstr (19, 1, "Margine sinistro:" );
   mvaddstr (19,56, "(numero spazi bianchi)");
   mvaddstr (20, 1, "Lunghezza pagina:" );
   mvaddstr (20,56, "(numero linee)");
   attrset (A_NORMAL);

   /* Messaggio di conferma per uscita dalla mappa:    */
   mvaddstr (23, 1, "Confermi la copia della stampa (S/N)?" );

   /* Loop per input di ciascun campo variabile         */
   /* Sono gestiti i campi freccie, PgDn                */
   attrset (A_REVERSE);
   for (;;) {
l1: ;
      move ( 5,30); c = inputtext(cp_hold, 1,'U', 'F');
      if ((strcmp(cp_hold, "S") != 0) && (strcmp(cp_hold, "N") != 0)
         && (strcmp(cp_hold, " ") != 0)) goto l1;
      if (c == KEY_UP) goto lfine; if (c == KEY_NPAGE) goto lfine;
l2: ;
      move ( 6,30); c = inputtext(cp_copie,2,'9', 'F');
      if (c == KEY_UP) goto l1;    if (c == KEY_NPAGE) goto lfine;
l3: ;
      move ( 7,30); c = inputtext(cp_compr,1,'U', 'F');
      if ((strcmp(cp_compr, "S") != 0) && (strcmp(cp_compr, "N") != 0)
         && (strcmp(cp_compr, " ") != 0))  goto l3;
      if (c == KEY_UP) goto l2;    if (c == KEY_NPAGE) goto lfine;
l4: ;
      move ( 8,30); c = inputtext(cp_enf,1,'U', 'F');
      if ((strcmp(cp_enf, "S") != 0) && (strcmp(cp_enf, "N") != 0)
         && (strcmp(cp_enf, " ") != 0))  goto l4;
      if (c == KEY_UP) goto l3;    if (c == KEY_NPAGE) goto lfine;
l5: ;
      move (10,30); c = inputtext(cp_font,20,'A', 'F');
      if (c == KEY_UP) goto l4;    if (c == KEY_NPAGE) goto lfine;
l6: ;
      move (11,30); c = inputtext(cp_pitch,2,'9', 'F');
      if (c == KEY_UP) goto l5;    if (c == KEY_NPAGE) goto lfine;
l7: ;
      move (12,30); c = inputtext(cp_lpi,2,'9', 'F');
      if (c == KEY_UP) goto l6;    if (c == KEY_NPAGE) goto lfine;
l8: ;
      move (13,30); c = inputtext(cp_duplex,1,'U', 'F');
      if ((strcmp(cp_duplex, "F") != 0) && (strcmp(cp_duplex, "L") != 0)
          && (strcmp(cp_duplex, "C") != 0) && (strcmp(cp_duplex, " ") != 0)) goto l8;
      if (c == KEY_UP) goto l7;    if (c == KEY_NPAGE) goto lfine;
l9: ;
      move (14,30); c = inputtext(cp_form,  1,'U', 'F');
      if ((strcmp(cp_form, "P") != 0) && (strcmp(cp_form, "L") != 0)
         && (strcmp(cp_form, " ") != 0)) goto l9;
      if (c == KEY_UP) goto l8;    if (c == KEY_NPAGE) goto lfine;
l10: ;
      move (15,30); c = inputtext(cp_source,1,'9', 'F');
      if (c == KEY_UP) goto l9;    if (c == KEY_NPAGE) goto lfine;
l11: ;
      move (16,30); c = inputtext(cp_binout,2,'9', 'F');
      if (c == KEY_UP) goto l10;   if (c == KEY_NPAGE) goto lfine;

l12: ;
      move (18,30); c = inputtext(cp_top,2,'9', 'F');
      if (c == KEY_UP) goto l11;   if (c == KEY_NPAGE) goto lfine;
l13: ;
      move (19,30); c = inputtext(cp_left,2,'9', 'F');
      if (c == KEY_UP) goto l12;   if (c == KEY_NPAGE) goto lfine;
l14: ;
      move (20,30); c = inputtext(cp_lpage,8,'9', 'F');
      if (c == KEY_UP) goto l13;   if (c == KEY_NPAGE) goto lfine;
lfine: ;

      move (23,40); c = inputtext(cp_yn,1,'U', 'F');
      if (c == KEY_UP) goto l14;
      if (strcmp(cp_yn, "Y") == 0) strcpy(cp_yn, "S");
      if ((strcmp(cp_yn, "S") == 0) || (strcmp(cp_yn, "N") == 0)) break;

   }
   attrset (A_NORMAL);
   if (strcmp(cp_yn, "N") == 0) return (int) cp_yn;

   /* Costruzione dei parametri corrispondenti del comando 'qprt' */
   if (cp_hold[0] == 'S')     strcat(cp_cmd, " -#h");
   if (atoi(cp_copie) > 0) {
      strcat(cp_cmd, " -N");
      strcat(cp_cmd, cp_copie);
   }
   if (atoi(cp_pitch) > 0) {
      strcat(cp_cmd, " -p");
      strcat(cp_cmd, cp_pitch);
   }
   if (atoi(cp_lpi) > 0) {
      strcat(cp_cmd, " -v");
      strcat(cp_cmd, cp_lpi);
   }
   if (atoi(cp_binout) > 0) {
      strcat(cp_cmd, " -=");
      strcat(cp_cmd, cp_binout);
   }
   if (atoi(cp_source) > 0) {
      strcat(cp_cmd, " -u");
      strcat(cp_cmd, cp_source);
   }
   if (atoi(cp_top) > 0) {
      strcat(cp_cmd, " -t");
      strcat(cp_cmd, cp_top);
   }
   if (atoi(cp_left) > 0) {
      strcat(cp_cmd, " -i");
      strcat(cp_cmd, cp_left);
   }
   if (atoi(cp_lpage) > 0) {
      strcat(cp_cmd, " -l");
      strcat(cp_cmd, cp_lpage);
   }
   if (cp_compr[0] == 'S')     strcat(cp_cmd, " -K+");
   if (cp_compr[0] == 'N')     strcat(cp_cmd, " -K!");
   if (cp_enf[0] == 'S')       strcat(cp_cmd, " -e+");
   if (cp_enf[0] == 'N')       strcat(cp_cmd, " -e!");
   if (cp_form[0] == 'P')      strcat(cp_cmd, " -z0");
   if (cp_form[0] == 'L')      strcat(cp_cmd, " -z1");
   if (cp_duplex[0] == 'F')    strcat(cp_cmd, " -Y0");
   if (cp_duplex[0] == 'L')    strcat(cp_cmd, " -Y1");
   if (cp_duplex[0] == 'C')    strcat(cp_cmd, " -Y2");
   if ((cp_font[0] != ' ') && (cp_font[1] != ' ')) {
      strcat(cp_cmd, " -s");
      strcat(cp_cmd, cp_font);
   }

/* mvaddstr (22, 1, cp_cmd );
   c = getch();  *    */

   return (int) cp_yn;
}


/* Estrae dettagli statistici (n. pagine, larghezza) dal file di stampa */

int get_stat(char *nomein)

{
   char c, *ptr, linea[256] ;
   long int i, l, k;

   filelen = 0; numlines = 0; numpag = 0; numctrll = 0; maxlenrec = 0;
   c = '\f';

   if((filepr = fopen(nomein,"r")) == NULL) {
      return(10) ;
   }
   if (stat(nomein,&statbuf) != 0) {
      fclose(filepr );
      return(11) ;
   }
   filelen = statbuf.st_size;

   l = 0;
   while (fgets(linea, 255, filepr) != NULL)  {
       numlines++; l++;
       k = strlen(linea);
       if (k > maxlenrec) maxlenrec = k;

       /* Conta anche piu' salti pagina per linea    */
       ptr = strchr (linea, c);
       while (ptr) {
          numctrll++; l = 0;
          ptr = strchr (ptr+1, c);
       }

       /* for (i = 0; i < k - 2; i++)
          if ( linea[i] == c ) {
             numctrll++; l = 0;
          }    */

       if ( l > linexpag ) {
          numpag++; l = 0;
       }
   }

   fclose (filepr);
   return(0);
}


/* Riempie la schermata con i dati dettagliati della stampa       */
/* Input: indice del vettore con dati stampa da visualizzare      */

void statout(int nrec)
{
   char s[8], stringa[80];

   int  i,l, col = 2, riga = 0;


   /* Scrive 2 linee orizzontali */
   attrset(A_NORMAL);
   /* attrset(A_ALTCHARSET); */
   for (i = 0 ; i < 79; i++) {
      mvaddch(0, i, chc0[1]);
      mvaddch(2, i, chc0[1]);
   }

   /* Scrive cornici e intestazioni     */
   mvaddstr(0, 1, "User: ");
   mvaddstr(0, 7, nomeuser);
   mvaddstr(0, 65, dateday);
   mvaddstr(0, 74, ora);
   attrset(A_NORMAL | A_REVERSE);
   mvaddstr(0, 30, " Gestione Stampe ");

   attrset (A_NORMAL | A_BOLD );
   mvaddstr ( 1, 1, lineablank );
   attrset (A_NORMAL);

      sprintf(stringa, "Coda:                  Stato: %9s %9s                  %4d stampe",
           qstatus[nworkq], qremstat[nworkq], totfileq);
      mvaddstr ( 1, 1, stringa);
      if (strcmp(pagprint, "       ") != 0) {
         sprintf(stringa, "Pag.:%s%%",pagprint);
         mvaddstr ( 1, 51, stringa);
      }
      attrset (A_REVERSE);
      mvaddstr ( 1, 7, workq);

   attrset (A_NORMAL | A_BOLD);
   mvaddstr ( 2, 1, "Informazioni dettagliate sulla stampa ");

   mvaddstr ( 4, 1, "Stampa n." );
   mvaddstr ( 4,35, "Stato" );
   mvaddstr ( 4,55, "Job number:" );
   mvaddstr ( 5, 1, "Nome stampa:");
   mvaddstr ( 6, 1, "Utente:" );
   mvaddstr ( 7, 1, "Data e Ora:");
   mvaddstr ( 8, 1, "Pagine:" );
   mvaddstr ( 8,35, "Salti pagina:" );
   mvaddstr ( 8,59, "Linee:" );
   mvaddstr ( 9, 1, "Colonne:" );
   mvaddstr (10, 1, "Priorita':" );
   mvaddstr (11, 1, "Lunghezza in bytes:  " );
   mvaddstr (12, 1, "Lunghezza in blocchi:" );
   mvaddstr (13, 1, "File fisico:" );
   mvaddstr (14, 1, "Destinatario:" );
   mvaddstr (15, 1, "Copie:" );

   i = nrec;
   attrset (A_NORMAL);
   sprintf  (s, "%d", nrec);
   mvaddstr ( 4,16, s);
   mvaddstr ( 4,45, filestat[i]);
   mvaddstr ( 4,68, filejob[i]);
   mvaddstr ( 5,16, filename[i]);
   mvaddstr ( 6,16, fileuser[i]);
   mvaddstr ( 7,16, filedate[i]);
   sprintf  (s, "%ld", 1 + numctrll + numpag);
   mvaddstr ( 8,16, s);
   sprintf  (s, "%ld", numctrll);
   mvaddstr ( 8,50, s);
   sprintf  (s, "%ld", numlines);
   mvaddstr ( 8,68, s);
   sprintf  (s, "%ld", maxlenrec-1);
   mvaddstr ( 9,16, s);

   mvaddstr (10,16, filepri[i]);
   sprintf  (s, "%ld", filelen);
   mvaddstr (11,22, s);

   mvaddstr (12,22, fileblk[i]);
   mvaddstr (13,16, filepath[i]);
   mvaddstr (14,16, fileuser1[i]);
   mvaddstr (15,16, filecpy[i]);

   mvaddstr (20, 1, "Premi <Invio> per continuare: " );

}


int main(int argc, char *argv[])

{

    int  linecurs = 1;    /* linea occupata dal cursore (in reverse)  */
    int  nrec = 1;        /* posizione del cursore nel vettore        */
    int  totline ;        /* totale elementi del vettore              */

    int i, j, k, l, n, c, uc;
    char stringa[MAXSTR], comando[MAXSTR];
    char *cp, *ptr, prec = ' ';

    strcpy (lineablank, "                                                                              ");
    strcpy(record, "");
    strcpy(workq, "");
    strcpy(titolo, " Gestione Stampe ");
    mask = A_NORMAL;

    data();       /* estrae data, ora giorno del mese e della settimana */

    #ifdef AIX
    /* Caratteri cornice singola per AIX      */
    chc2[0] = 'x';    chc2[1] = 'o';     chc2[2] = 'l';
    chc2[3] = 'k';    chc2[4] = 'm';     chc2[5] = 'j';
    chc2[6] = 't';    chc2[7] = 'u';     chc2[8] = 'w';    chc2[9] = 'v';
    #else  /* Caratteri cornice per ncurses Linux   */
    chc2[0] = ACS_VLINE;    chc2[1] = ACS_HLINE;   chc2[2] = ACS_ULCORNER;
    chc2[3] = ACS_URCORNER; chc2[4] = ACS_LLCORNER;  chc2[5] = ACS_LRCORNER;
    #endif
    for (i = 0; i < 10 ; i++) chc0[i] = chc1[i];


    /* Esamina eventuali parametri in input su linea comandi    */
    if (  (strcmp(argv[1],"?") == 0) || (strcmp(argv[1],"-h") == 0) ||
                       (strcmp(argv[1],"-?") == 0) ) {
       hlp_par();
       exit(0);
    }
    strcpy(elencocmd,"");
    strcpy(elencoque,"");
    for (i = 1 ; i < argc ; i++) {
       if ( strncmp(argv[i],"-S",2) == 0) {  /* e' richiesto full   */
          fulflag = 'S';
          goto loop1;
       }
       if ( strncmp(argv[i],"-X3",3) == 0) {  /* compatibile AIX 3.2.5 */
          X3flag = 'S';
          goto loop1;
       }
       if ( strncmp(argv[i],"-T",2) == 0) {  /* cambia il titolo finestra */
          if (strlen(argv[i]) > 2)
             strcpy(titolo, argv[i]+2);
          else prec = 'T';
          goto loop1;
       }
       if ( strncmp(argv[i],"-L",2) == 0) {  /* e' richiesto il log */
          logflag = 'S';
          if (strlen(argv[i]) > 2)
             strcpy(filelog, argv[i]+2);
          else prec = 'L';
       }
       else {
          if ( strncmp(argv[i],"-!",2) == 0) {  /* comandi proibiti */
             if (strlen(argv[i]) > 2)
                strcpy(elencocmd, argv[i]+2);
             else prec = '!';
          }
          else {
             if ( strncmp(argv[i],"-P",2) == 0) {  /* code permesse */
                queflag = 'S';
                if (strlen(argv[i]) > 2)
                   strcpy(elencoque, argv[i]+2);
                else prec = 'P';
             }
             else { 
                if ( strncmp(argv[i],"-B",2) == 0) { /* backend permessi */
                   bflag = 'S';
                   if (strlen(argv[i]) > 2)
                      strcpy(elencobak, argv[i]+2);
                   else prec = 'B';
                }
                else { /* e' stato lasciato un blank dopo il par. precedente */
                   if (prec == ' ') {
                      printf ("\nISM - Parametro errato.  Usa -? per help.\n");
                      exit(0);
                   }
                   else {
                      if (prec == 'L') strcpy(filelog, argv[i]);
                      if (prec == 'P') strcpy(elencoque, argv[i]);
                      if (prec == 'B') strcpy(elencobak, argv[i]);
                      if (prec == '!') strcpy(elencocmd, argv[i]);
                      if (prec == 'T') strcpy(titolo, argv[i]);
                      prec = ' ';
                   }
                }
             }
          }
       }
    loop1:  ;

    }

    if ( bflag == 'S' ) { /*  Separa i vari backend scritti nel parametro */
       for (nbackend = 0; nbackend < 6; nbackend ++) {
          if ((ptr = strchr(elencobak, ',')) != NULL ) {
             strncpy(backend[nbackend],elencobak,ptr-elencobak);
             backend[nbackend][ptr-elencobak] = '\0';
             strcpy(elencobak,ptr+1);
          }
          else {
             strcpy(backend[nbackend],elencobak);
             break;
          }
       }
    }

    for (i = 0; i < strlen(elencocmd) ; i++)
        elencocmd[i] = toupper(elencocmd[i]) ;
    if (strlen(elencoque) <= 0) queflag = ' ';

    strcpy (nomeuser, getenv("LOGNAME"));
    strcpy (homedir,  getenv("HOME"));
    /* strcpy (hostname, getenv("HOSTNAME")); */

    if ( logflag == 'S' ) {    /*  Apre il file di log  */
       if ( (lfp = fopen(filelog, "a+t")) == NULL) {
          fprintf(stderr,"ISM - Errore in apertura log %s.\n",filelog);
          exit (97);
       }
       else {
          sprintf (stringa, "\nInizio ISM - User %s  - %s %s \n",
                  nomeuser, dateday, ora);
          if (fputs(stringa, lfp) == EOF ) {
             fprintf(stderr,"ISM - Errore in scrittura log %s.\n",filelog);
             exit (90);
          }
       }
    }

    /* Nome file temporaneo contenente l'output di "lpstat "   */
    strcpy (filetemp, homedir);
    strcat (filetemp, "/tmp~");
    strcat (filetemp, minsec);
    strcat (filetemp, ".enq");

    /* Ottiene elenco code nel sistema   */
    getqueues();
    if (totq <= 0) {     /* termina se non esistono code    */
       system (CLEAR);
       printf ("\n\n\n\n\n\n");
       printf ("ISM - In questo sistema non sono presenti code di stampa\n");
       printf ("      oppure non e' consentita la loro gestione a questo utente.\n\n");
       printf ("\n\n\n\n\n\n");
       goto fine1;
    }

    /* Inizializza curses e variabili    */
    initscr();
    cbreak();
    noecho();
    keypad (stdscr, TRUE);
    clear();

    tipovis = 'C';      /* la schermata iniziale e' l'elenco delle Code */
    if (totq == 1) {    /* a meno che non ci sia una sola coda  */
       strcpy(workq, qname[1]);
       nworkq = 1;
       getspool(workq);
       tipovis = 'J';
    }
    else  
       if (fulflag == 'S') {
          for (i = 1; i <= totq; i++) {
             strcpy(workq, qname[i]);
             nworkq = i;
             getspool(workq);

          }
    }


    nlinepag = 20;  /* numero linee per pagina       */
    linecurs = 1;   /* posizione linea del cursore   */
    nrec = 1;       /* indice dei vettori contenenti i dati */

    /* Main loop di rilevamento tasti premuti e azioni conseguenti  */
    for (;;) {
       RC=0;

       /* Rigenera elenco stampe su file temporaneo  se necessario */
       if ((tipovis != 'C' ) && (rigen == 'S')) {
          getspool(workq);  rigen = ' ';
       }
       if (tipovis == 'C') totline = totq; else totline = totfileq;
       if (nrec > totline) {
          linecurs = linecurs - (nrec - totline);
          nrec = totline;
       }
       if (linecurs < 1) { linecurs = 1; nrec = 1; }
       if (totline == 0) { linecurs = 0; nrec = 0; }


       /* Visualizza elenco code / stampe       */
       noecho();
       videoout(linecurs, nrec);
       refresh();

       /* Attende tasto premuto dall'utente   */
       c = getch();
       c = toupper(c);

       /* Esegue eventuale comando associato al tasto premuto   */

       if (c == 'A') {      /* aggiorna visualizzazione code     */
          rigen = 'S';
          clear();
          goto loop;
       }
       if ((c == KEY_NPAGE) || (c == 'F')) {
          if (nrec+nlinepag-1 < totline) {
             nrec = nrec+19;
             if (nrec+linecurs > totline) linecurs = 1;
          }
          else linecurs = totline - nrec - 1;
          if (linecurs > nrec) linecurs = nrec;
          clear();
          goto loop;
       }
       if ((c == KEY_PPAGE) || (c == 'B')) {
          if ((nrec > 1) && (nrec > nlinepag) ) {
             nrec = nrec - nlinepag;
             if (linecurs > nrec) linecurs = nrec;
          }
          else {
             linecurs = 1;
             nrec = 1;
          }
          clear();
          goto loop;
       }
       if ((c == KEY_DOWN) || (c == KEY_SR) || (c == 'J')) {
          if ((linecurs >= nlinepag) && (nrec < totline))
             nrec++;
          else
             if ((linecurs < nlinepag) && (nrec < totline)) {
                 linecurs++; nrec++;
             }
          goto loop;
       }
       if ((c == KEY_UP) || (c == KEY_SF) || (c == 'K')) {
          if (linecurs > 1 ) { linecurs--; nrec--; }
          else if (nrec > 1) nrec--;

          goto loop;
       }
       if ( (c == KEY_HOME) || (c == KEY_A1) ) {
          nrec = nrec - linecurs + 1;
          linecurs = 1;
          clear();

          goto loop;
       }
       if ((c == '\n') || (c == 'V')) {    /* se e' premuto ENTER */
          if (tipovis == 'C') {
             strcpy(workq, qname[nrec]);
             nworkq = nrec;
             getspool(workq);
             tipovis = 'J';
             linecurs = 1;   /* posizione linea del cursore   */
             nrec = 1;
          }
          else {
             if (strchr(elencocmd, 'V') == NULL) {
                if (c == 'V') strcpy (comando, PAGER2);    /* view   */
                else strcpy (comando, PAGER1);       /* most   */
                strcat (comando, " ");
                strcat (comando, filepath[nrec]);
                if (strlen(filepath[nrec]) > 46) strcat (comando, "*");
                system (CLEAR);  
                system (comando);
             }
          }
          clear();
          goto loop;
       }
       if ((c == KEY_F(1)) || (c == '?') ) {  /* Help screen   */
          hlp_scr();
          goto loop;
       }
       if ((c == KEY_F(3)) || (c == KEY_F(4)) || (c == 'W') ) {  /* Schermata prec. */
          if (tipovis == 'J') tipovis = 'C';
          else break;
          linecurs = 1;   /* posizione linea del cursore   */
          nrec = 1;
          clear();
          if (totq == 1) break;
          goto loop;
       }
       if ( (c == 'C') && (tipovis != 'C') ){    /* Copia file */
          if (strchr(elencocmd, c) == NULL) {
             strcpy ( stringa, " Nome del file in cui copiare la stampa: ");
             display23 ( stringa );
             strcpy(stringa,"");
             move (23, 42);
             inputtext(stringa, 38, ' ', ' ');
             if (strlen(stringa) == 0) goto loop;
             strcpy (comando, "cp ");
             strcat (comando, filepath[nrec]);
             strcat (comando, " ");
             strcat (comando, stringa);
             move (23, 0); refresh();
             if ((RC = system (comando)) != 0) {
                display23(" Errore in Copia           (premi ENTER) ");
                c = getch();
             }
             if ( logflag == 'S' ) {
                sprintf (stringa, "%s | %s  | RC = %d \n",
                   filename[nrec], comando, RC);
                fputs(stringa, lfp);
             }
             goto loop;
          }
       }
       if ( c == 'U') {    /* Mette in UP la coda     */
          if (strchr(elencocmd, c) == NULL) {
             if (tipovis == 'C') {
                strcpy(workq, qname[nrec]);
                nworkq = nrec;
                tipovis = 'J';
             }
             strcpy (comando, "enq -P ");
             strcat (comando, workq);
             strcat (comando, " -U");
             if ((RC = system (comando)) != 0) {
                display23(" Errore in Up Device coda       (premi ENTER) ");
                c = getch();
             }
             if ( logflag == 'S' ) {
                sprintf (stringa, "| %s  | RC = %d \n", comando, RC);
                fputs(stringa, lfp);
             }
             sleep(1);    /* aspetta 1 secondo prima di rigenerare */
             rigen = 'S';
             clear();
             goto loop;
          }
       }
       if ( c == 'D')   {           /* Mette in DOWN la coda     */
          if (strchr(elencocmd, c) == NULL) {
             if (tipovis == 'C') {
                strcpy(workq, qname[nrec]);
                nworkq = nrec;
                tipovis = 'J';
             }
             strcpy (comando, "enq -P ");
             strcat (comando, workq);
             strcat (comando, " -K");
             if ((RC = system (comando)) != 0) {
                display23(" Errore in Down Device coda       (premi ENTER) ");
                c = getch();
             }
             if ( logflag == 'S' ) {
                sprintf (stringa, "| %s  | RC = %d \n", comando, RC);
                fputs(stringa, lfp);
             }
             sleep(1);    /* aspetta 1 secondo prima di rigenerare */
             rigen = 'S';
             clear();
             goto loop;
          }
       }
       if ( ((c == 'X') || (c == KEY_DC)) && (tipovis != 'C')) {
          if (strchr(elencocmd, c) == NULL) {
             /* Cancellazione stampa    */
             if (tagged > 0 ) sprintf (stringa," Confermi la cancellazione di %d stampe", tagged);
             else sprintf (stringa, " Confermi la cancellazione di %s", filename[nrec]);
             strcat ( stringa, " ? (s/N) ");
             display23 ( stringa );
             c = getch();
             if ((toupper(c) == 'S') || (toupper(c) == 'Y')) {
                move (23, 0); refresh();
                for (j = 1; j <= totline; j++) {
                   if ((j == nrec) || (filetag[j] != ' ')) {
                      filetag[j] = ' ';
                      strcpy (comando, "enq -P ");
                      strcat (comando, workq);
                      strcat (comando, " -x");
                      strcat (comando, filejob[j]);
                      if ((RC = system (comando)) != 0) {
                         display23(" Errore in Cancellazione        (premi ENTER) ");
                         c = getch();
                      }
                      if ( logflag == 'S' ) {
                         sprintf (stringa, "%s | %s  | RC = %d \n",
                            filename[j], comando, RC);
                         fputs(stringa, lfp);
                      }
                   }
                }
                tagged = 0;
                sleep(1);    /* aspetta 1 secondo prima di rigenerare */
                rigen = 'S';
                clear();
                c = ' ';
             }
          }
       }
       if ( (c == 'Z') && (tipovis != 'C')) {
          if (strchr(elencocmd, c) == NULL) {
             /* Cancellazione stampa    */
             strcpy ( stringa, " Confermi la cancellazione di TUTTA la ");
             strcat ( stringa, workq);
             strcat ( stringa, " ? (s/N) ");
             display23 ( stringa );
             c = getch();
             if ((toupper(c) == 'S') || (toupper(c) == 'Y')) {
                strcpy (comando, "enq -P ");
                strcat (comando, workq);
                strcat (comando, " -X");
                move (23, 0); refresh();
                if ((RC = system (comando)) != 0) {
                   display23(" Errore in Cancellazione        (premi ENTER) ");
                   c = getch();
                }
                if ( logflag == 'S' ) {
                   sprintf (stringa, "%s | %s  | RC = %d \n",
                      filename[nrec], comando, RC);
                   fputs(stringa, lfp);
                }
                rigen = 'S';
                sleep(2);    /* aspetta 2 secondi prima di rigenerare */
                clear();
                c = ' ';
             }
          }
       }
       if (( c == 'P') && (tipovis != 'C') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Cambio Priorita' della stampa */
             if (tagged > 0 ) sprintf (stringa," Nuova priorita' di %d stampe", tagged);
             else sprintf (stringa, " Nuova priorita' di %s", filename[nrec]);
             strcat (stringa, " (tra 1 e 30): ");
             display23 ( stringa );
             strcpy ( stringa, "" );
             move (23, 50);
             inputtext(stringa, 2, '9', ' ');
             if (strlen(stringa) == 0) goto loop;
             if ((atoi(stringa) > 30) || (atoi(stringa) < 1)) goto loop;
             move (23, 0); refresh();
             for (j = 1; j <= totline; j++) {
                if ((j == nrec) || (filetag[j] != ' ')) {
                   filetag[j] = ' ';
                   strcpy (comando, "enq -P ");
                   strcat (comando, workq);
                   strcat (comando, " -a");
                   strcat (comando, stringa);
                   strcat (comando, " -#");
                   strcat (comando, filejob[j]);
                   if ((RC = system (comando)) != 0) {
                      display23(" Errore in cambio priorita'     (premi ENTER) ");
                      c = getch();
                   }
                   if ( logflag == 'S' ) {
                      sprintf (stringa, "%s | %s  | RC = %d \n",
                         filename[j], comando, RC);
                      fputs(stringa, lfp);
                   }
                }
             }
             rigen = 'S';
             c = ' ';
             tagged = 0;
             sleep(1);
          }
       }
       if (( c == 'S') && (tipovis != 'C') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Spostamento della stampa in un'altra coda */
             if (tagged > 0 ) sprintf (stringa," Nuova coda in cui spostare %d stampe", tagged);
             else sprintf (stringa, " Nuova coda in cui spostare %s", filename[nrec]);
             display23 ( stringa );
             strcpy ( stringa, "" );
             move (23, 45);
             inputtext(stringa, 18, ' ', ' ');
             if (strlen(stringa) == 0) goto loop;
             move (23, 0); refresh();
             for (j = 1; j <= totline; j++) {
                if ((j == nrec) || (filetag[j] != ' ')) {
                   filetag[j] = ' ';
                   strcpy (comando, "enq -P ");
                   strcat (comando, workq);
                   strcat (comando, " -Q");
                   strcat (comando, stringa);
                   strcat (comando, " -#");
                   strcat (comando, filejob[j]);
                   if ((RC = system (comando)) != 0) {
                      display23(" Errore in Spostamento stampa   (premi ENTER) ");
                      c = getch();
                   }
                   if ( logflag == 'S' ) {
                      sprintf (stringa, "%s | %s  | RC = %d \n",
                         filename[j], comando, RC);
                      fputs(stringa, lfp);
                   }
                }
             }
             rigen = 'S';
             clear();
             sleep(1);
             tagged = 0;
             c = ' ';
          }
       }
       if (( c == 'H') && (tipovis != 'C') && (X3flag != 'S') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Congelamento  della stampa */
             for (j = 1; j <= totline; j++) {
                if ((j == nrec) || (filetag[j] != ' ')) {
                   filetag[j] = ' ';
                   strcpy (comando, "enq -P ");
                   strcat (comando, workq);
                   strcat (comando, " -h -#");
                   strcat (comando, filejob[j]);
                   move (23, 0); refresh();
                   if ((RC = system (comando)) != 0) {
                      display23(" Errore in congelamento stampa     (premi ENTER) ");
                      c = getch();
                   }
                   if ( logflag == 'S' ) {
                      sprintf (stringa, "%s | %s  | RC = %d \n",
                         filename[j], comando, RC);
                      fputs(stringa, lfp);
                   }
                }
             }
             rigen = 'S';
             c = ' ';
             sleep(1);
             tagged = 0;
          }
       }
       if (( c == 'R') && (tipovis != 'C') && (X3flag != 'S') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Rilascio  della stampa */
             for (j = 1; j <= totline; j++) {
                if ((j == nrec) || (filetag[j] != ' ')) {
                   filetag[j] = ' ';
                   strcpy (comando, "enq -P ");
                   strcat (comando, workq);
                   strcat (comando, " -p -#");
                   strcat (comando, filejob[j]);
                   move (23, 0); refresh();
                   if ((RC = system (comando)) != 0) {
                      display23(" Errore in rilascio stampa     (premi ENTER) ");
                      c = getch();
                   }
                   if ( logflag == 'S' ) {
                      sprintf (stringa, "%s | %s  | RC = %d \n",
                         filename[j], comando, RC);
                      fputs(stringa, lfp);
                   }
                }
             }
             rigen = 'S';
             clear();
             c = ' ';
             sleep(1);
             tagged = 0;
          }
       }
       if (( c == 'T') && (tipovis != 'C') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Ripartenza stampa da pagina prefissata */
             strcpy (pagini, "");
             strcpy (pagfin, "");
             strcpy (stringa, " Ripartenza stampa ");
             strcat (stringa, filename[nrec]);
             strcat (stringa, " da pagina: ");
             display23 ( stringa );
             strcpy ( stringa, "" );
             move (23, 50);
             inputtext(stringa, 20, '9', ' ');
             if (strlen(stringa) == 0) goto loop;
             sscanf(stringa, "%s %s", &pagini, &pagfin);
             display23 ( "Vuoi cambiare le caratteristiche di stampa delle pagine selezionate? (s/n)");
             c = getch();
             if ((toupper(c) == 'S') || (toupper(c) == 'Y')) {
                if (chg_stampa(nrec) == 'N') goto loop;
             }
             if (X3flag != 'S') {
                strcpy (comando, "enq -P ");
                strcat (comando, workq);
                strcat (comando, " -h -#");
                strcat (comando, filejob[nrec]);
                move (23, 0); refresh();
                if ((RC = system (comando)) != 0) {
                   display23(" Errore in ripartenza stampa     (premi ENTER) ");
                   c = getch();
                }
                if ( logflag == 'S' ) {
                   sprintf (stringa, "%s | %s  | RC = %d \n",
                      filename[nrec], comando, RC);
                   fputs(stringa, lfp);
                }
             }
             strcpy (comando, "pt -i");
             strcat (comando, pagini);
             if (strcmp(pagfin, "") != 0) {
                strcat (comando, " -f");
                strcat (comando, pagfin);
             }
             strcat (comando, " ");
             strcat (comando, filepath[nrec]);
             if (strlen(filepath[nrec]) > 46) strcat (comando, "*");
             strcat (comando, "|qprt -P");
             strcat (comando, workq);
             strcat (comando, " -T\"Rst");
             strcat (comando, filejob[nrec]);
             strcat (comando, "_");
             strcat (comando, pagini);
             strcat (comando, "\"");
             strcat (comando, cp_cmd);

             move (23, 0); refresh();
             if ((RC = system (comando)) != 0) {
                display23(" Errore in ripartenza stampa     (premi ENTER) ");
                c = getch();
             }
             if ( logflag == 'S' ) {
                sprintf (stringa, "%s | %s  | RC = %d \n",
                   filename[nrec], comando, RC);
                fputs(stringa, lfp);
             }
             move (23, 0); refresh();
             sleep(1);    /* aspetta 1 secondo prima di rigenerare */
             rigen = 'S';
             clear();
             c = ' ';
          }
       }
       if ( c == '/') {    /* Posiziona a inizio videata */
          linecurs = 1;
          clear();
          goto loop;
       }
       if ( c == '!') {    /* Lancia comando in shell Unix */
          if (strchr(elencocmd, c) == NULL) {
             display23 ( " SHELL $ ");
             strcpy ( comando, "" );
             move (23, 10);
             inputtext(comando, 68, ' ', ' ');
             if (strlen(comando) == 0) goto loop;
             strcpy (stringa, "echo ");
             strcat (stringa, comando);
             system (CLEAR);
             system (stringa);
             RC = system (comando);
             if ( logflag == 'S' ) {
                sprintf (stringa, "! %s  | RC = %d \n", comando, RC);
                fputs(stringa, lfp);
             }
             c = getch();
             clear();
             goto loop;
          }
       }
       if ((c == 'I') && (tipovis != 'C') && (nrec > 0) && (strchr(elencocmd, c) == NULL)) {
           /* Richiede informazioni sulla stampa */
          clear();
          RC=get_stat(filepath[nrec]);
          statout(nrec);
          refresh();
          c = getch();
          clear();
          goto loop;
       }
       if (( c == 'G') && (tipovis != 'C') ) {
          if (strchr(elencocmd, c) == NULL) {
             /* Duplicazione stampa                    */
             rigen = 'S';
             chg_stampa(nrec) ;
             if (strcmp(cp_yn, "N") == 0 ) goto loop;
             strcpy (comando, "cat ");
             strcat (comando, filepath[nrec]);
             if (strlen(filepath[nrec]) > 46) strcat (comando, "*");
             strcat (comando, "|qprt -P");
             strcat (comando, workq);
             strcat (comando, " -T\"Cpy");
             strcat (comando, filejob[nrec]);
             strcat (comando, "\"");
             strcat (comando, cp_cmd);

             move (23, 0); refresh();
             if ((RC = system (comando)) != 0) {
                display23(" Errore in Copia           (premi ENTER) ");
                c = getch();
             }
             if ( logflag == 'S' ) {
                sprintf (stringa, "%s | %s  | RC = %d \n",
                   filename[nrec], comando, RC);
                fputs(stringa, lfp);
             }
             sleep(1);    /* aspetta 1 secondo prima di rigenerare */
             rigen = 'S';
             clear();
             c = ' ';
          }
       }

       if ((c == ' ') && (tipovis != 'C') && (nrec > 0) && (strchr(elencocmd, 'B') == NULL)) {
          if (filetag[nrec] == ' ') {
             filetag[nrec] = '>';
             tagged ++;
          }
          else {
             filetag[nrec] = ' ';
             tagged --;
          }
       }

       if ( (c == 'Y') ) {
          for (j = 1; j <= totline; j++) filetag[j] = ' ';
          tagged = 0;
          clear();
       }

       if ( (c == 'Q') || (c == KEY_F(10)) || (c == KEY_BREAK) )
          break;

loop: ;
    }


fine: ;
    move(numrighe-1,0);
    refresh();
    endwin();
fine1: ;
    if (logflag == 'S') {
       data();   /* estrae data, ora giorno del mese e della settimana */
       sprintf (stringa, "Fine ISM - User %s  - %s %s \n",
               nomeuser, dateday, ora);
       fputs(stringa, lfp);
       fclose(lfp);
    }
    /* Cancella file temporanei              */
    strcpy(comando, "rm ");
    strcat(comando,filetemp);
    if ( system(comando) != 0) {
       fprintf(stderr,"ISM - Errore in comando: %s.\n", comando);
       exit (98);
    }

    return 0;
}

/* Help screen - interattivo           */

void hlp_scr()
{
     char c;

     werase(stdscr);
     attrset (A_NORMAL );

     mvaddstr(0, 26, " INTERACTIVE  SPOOL  MANAGER");
     mvaddstr(1, 22, " Gestione interattiva code di stampa ");
     mvaddstr(2, 22, " v. 1.1 - (C) Piero Ronchi 10/09/1998");

     mvaddstr(4, 35, "T A S T I");
     mvaddstr(5, 1, "frecce-j/k    Posiziona su coda/stampa");
     mvaddstr(6, 1, "PgUp/PgDn-F/B Pagina prec./successiva");
     mvaddstr(7, 1, "ENTER         Visualizza coda/stampa");
     mvaddstr(8, 1, "q - F10       Fine programma");

     mvaddstr(5, 41, "HOME     Primo file dello schermo");
     mvaddstr(6, 41, "END      Ultimo file dello schermo");
     mvaddstr(7, 41, "w - F3   Schermata precedente code");
     mvaddstr(8, 41, "/        Posiziona riga a inizio video");

     mvaddstr( 9, 33, "C O M A N D I");
     mvaddstr(10, 1, "U     Apre coda   (Up)     ");
     mvaddstr(11, 1, "D     Chiude coda (Down)  ");
     mvaddstr(12, 1, "A     Aggiorna la videata");
     mvaddstr(13, 1, "X     Cancella la stampa selezionata");
     mvaddstr(14, 1, "H     Congela una stampa (Hold)   ");
     mvaddstr(15, 1, "R     Rilascia una stampa congelata ");
     mvaddstr(16, 1, "S     Sposta da una coda all'altra");
     mvaddstr(17, 1, "G     Duplica con cambio caratteristiche");
     mvaddstr(18, 1, "<spazio> Selezione per comandi multipli");

     mvaddstr(10, 42, "Z     Cancella TUTTE le stampe");
     mvaddstr(11, 42, "P     Cambia la priorita' di stampa ");
     mvaddstr(12, 42, "V     Visualizza la stampa");
     mvaddstr(13, 42, "I     Chiede informazioni estese");
     mvaddstr(14, 42, "T     Riprende stampa a partire da");
     mvaddstr(15, 42, "C     Copia la stampa su un file  ");
     mvaddstr(16, 42, "!     Esegue comando in shell Unix");
     mvaddstr(17, 42, "Y     Toglie selez. comandi multipli");

     mvaddstr(20, 1, "In visualizzazione stampe usa le freccie per spostarti e inoltre:");
     mvaddstr(21, 1, "       B    va a fine stampa             T    va a inizio stampa");
     mvaddstr(22, 1, "       /    ricerca una parola particolare all'interno della stampa");

     attrset (A_BOLD );
     mvaddstr(23, 28, "Premi ENTER per ritornare");
     attrset (A_NORMAL );

     refresh();
     c = getch();

     clear();
}


/* Help screen parametri - batch                 */

void hlp_par()
{
printf("\nISM - Interactive Spool Manager ---------------------------- Piero Ronchi 9/98");
printf("\nUtility per la gestione full screen delle code di stampa in ambiente AIX 4 \n");
printf("\nUso:  ism  [-Pelenco_code] [-!comandi_non_permessi] [-L[logfile]] [-Bbackend] \n");
printf("\nSenza parametri si puo' dare qualunque comando su qualunque coda.");
printf("\nCon i seguenti parametri si restringe all'utente l'uso di code e comandi:" );
printf("\n      -Pcoda1,coda2,coda3...   Permette la gestione solo delle code elencate");
printf("\n      -!                       le lettere che seguono il '!' impediscono:");
printf("\n              z     cancellazione di TUTTE le stampe di una coda");
printf("\n              x     cancellazione di qualunque stampa o coda ");
printf("\n              u     comando apertura coda (Up)               ");
printf("\n              d     comando chiusura coda (Down)             ");
printf("\n              p     cambio priorita' della stampa            ");
printf("\n              v     visualizzazione file di stampa           ");
printf("\n              c     copia file di stampa in un'altro file    ");
printf("\n              h     congelamento (hold) stampa               ");
printf("\n              r     rilascio     stampa                      ");
printf("\n              t     ripartenza a partire da pagina determinata");
printf("\n              g     duplicazione stampa con cambio caratteristiche");
printf("\n              s     spostamento stampa da una coda all'altra ");
printf("\n              b     selezione multipla con barra spaziatrice ");
printf("\n      -Bbackend1,backend2,...  Gestisce solo le code coi backend specificati");
printf("\n      -L                       produce un log, di default chiamato 'ism.log'");
printf("\n      -S                       raccoglie immediatamente i dati sulle code   ");
printf("\n      -T\" Titolo finestra \"   ");
printf("\n");   
}
